@@ -105,7 +105,7 @@ class FileLoader():
                 if img == None:
                     if not quiet:
                         print(f'Loading {file}')
-                    img = self.loadSingleImage(basepath/file,coords=local_coords, return_q = output_qxy,image_slice=image_slice,use_cached_md=True)
+                    img = self.loadSingleImage(basepath/file,coords=local_coords, return_q = output_qxy,image_slice=image_slice,use_cached_md=False)
                     # this is a dataarray with dims ['pix_x', 'pix_y']+attrs (standardized)
                     # e.g. generated by img = xr.DataArray(img,dims=['pix_x','pix_y'],
                     #      coords={},attrs=headerdict)
@@ -152,7 +152,10 @@ class FileLoader():
         #    list(dest_coords.keys()
         #    )
         #)
-        index = pd.MultiIndex.from_arrays(vals,names=keys)
+        try:
+            index = pd.MultiIndex.from_arrays(vals,names=keys)
+        except ValueError as e:
+            raise ValueError('This load found files, but none were deemed loadable.\nThis usually means that you set a file_filter or md_filter that was too restrictive, or your directory is wrong.\nCheck and rerun') from e
         index.name = 'system'
         if output_qxy:
             #come up with destination qx/qy here
