@@ -17,6 +17,11 @@ try:
 except ImportError:
     MACHINE_HAS_CUDA=False
     warnings.warn('Could not import CuPy or ndigpu.  If you expect this machine to support CuPy, check dependencies.  Falling back to scikit-image/numpy CPU integration.',stacklevel=2)
+try:
+    import dask.array as da
+    import dask
+except ImportError:
+    warnings.warn('Failed to import Dask, if Dask reduction is desired install pyhyperscattering[performance]',stacklevel=2)
 
 
 class WPIntegrator():
@@ -24,7 +29,7 @@ class WPIntegrator():
     Integrator for qx/qy format xarrays using skimage.transform.warp_polar or a custom cuda-accelerated version, warp_polar_gpu
     '''
     
-    def __init__(self,return_cupy=False,force_np_backend=False):
+    def __init__(self,return_cupy=False,force_np_backend=False,use_chunked_processing=False):
         '''
         Args:
             return_cupy (bool, default False): return arrays as cupy rather than numpy, for further GPU processing
@@ -36,6 +41,7 @@ class WPIntegrator():
             self.MACHINE_HAS_CUDA = False
             
         self.return_cupy = return_cupy
+        self.use_chunked_processing=use_chunked_processing
     
     def warp_polar_gpu(self,image, center=None, radius=None, output_shape=None, **kwargs):
         """
@@ -96,19 +102,19 @@ class WPIntegrator():
                     .interp(qy=0)
                     .data)        
         try:
-            stacked_axis = list(img.indexes.keys())
+            stacked_axis = list(img.coords)
             stacked_axis.remove('qx')
             stacked_axis.remove('qy')
-            assert len(stacked_axis)==1, "More than one axis left after removing qx and qy, not sure how to handle"
+            assert len(stacked_axis)==1, f"More than one axis left ({stacked_axis}) after removing qx and qy, not sure how to handle"
             stacked_axis = stacked_axis[0]
             system_to_integ = img.__getattr__(stacked_axis)
         except AttributeError:
             pass
         
         if self.MACHINE_HAS_CUDA:
-            TwoD = self.warp_polar_gpu(img_to_integ,center=(center_x,center_y))
+            TwoD = self.warp_polar_gpu(img_to_integ,center=(center_x,center_y), radius = np.nanmax(img_to_integ.shape))
         else:
-            TwoD = skimage.transform.warp_polar(img_to_integ,center=(center_x,center_y))
+            TwoD = skimage.transform.warp_polar(img_to_integ,center=(center_x,center_y), radius = np.nanmax(img_to_integ.shape))
 
         
         qx = img.qx
@@ -126,7 +132,21 @@ class WPIntegrator():
             return xr.DataArray(TwoD,dims=['chi','q'],coords={'q':q,'chi':chi},attrs=img.attrs)
 
 
-    def integrateImageStack(self,data):
+    def integrateImageStack(self,img_stack,method=None,chunksize=None):
+        '''
+        
+        '''
+        if (self.use_chunked_processing and method is None) or method=='dask':
+            func_args = {}
+            if chunksize is not None:
+                func_args['chunksize'] = chunksize
+            return self.integrateImageStack_dask(img_stack,**func_args)
+        elif (method is None) or method == 'legacy':
+            return self.integrateImageStack_legacy(img_stack)
+        else:
+            raise NotImplementedError(f'unsupported integration method {method}')
+
+    def integrateImageStack_legacy(self,data):
         #int_stack = img_stack.groupby('system').map(self.integrateSingleImage)   
         #return int_stack
         indexes = list(data.indexes.keys())
@@ -152,4 +172,57 @@ class WPIntegrator():
                 warnings.warn('Your index set contains duplicate conditions.  This is not supported and may not work.  Try adding additional coords to separate image conditions',stacklevel=2)
         
             data_int = data.groupby('pyhyper_internal_multiindex',squeeze=False).progress_apply(self.integrateSingleImage).unstack('pyhyper_internal_multiindex')
-        return data_int
\ No newline at end of file
+        return data_int
+    
+    
+    def integrateImageStack_dask(self,data,chunksize=5):
+        #int_stack = img_stack.groupby('system').map(self.integrateSingleImage)   
+        #return int_stack
+        indexes = list(data.indexes.keys())
+        try:
+            indexes.remove('pix_x')
+            indexes.remove('pix_y')
+        except ValueError:
+            pass
+        try:
+            indexes.remove('qx')
+            indexes.remove('qy')
+        except ValueError:
+            pass
+        
+        if len(indexes) == 1:
+            if data.__getattr__(indexes[0]).to_pandas().drop_duplicates().shape[0] != data.__getattr__(indexes[0]).shape[0]:
+                warnings.warn(f'Axis {indexes[0]} contains duplicate conditions.  This is not supported and may not work.  Try adding additional coords to separate image conditions',stacklevel=2)
+            
+            fake_image_to_process = data.isel(**{indexes[0]:0},drop=False)
+            data = data.chunk({indexes[0]:chunksize})
+        else:
+            #some kinda logic to check for existing multiindexes and stack into them appropriately maybe
+            data = data.stack({'pyhyper_internal_multiindex':indexes})
+            if data.pyhyper_internal_multiindex.to_pandas().drop_duplicates().shape[0] != data.pyhyper_internal_multiindex.shape[0]:
+                warnings.warn('Your index set contains duplicate conditions.  This is not supported and may not work.  Try adding additional coords to separate image conditions',stacklevel=2)
+                
+            fake_image_to_process = data.isel(**{'pyhyper_internal_multiindex':0},squeeze=False)
+            data = data.chunk({'pyhyper_internal_multiindex':chunksize})
+        coord_dict = {}
+        shape = tuple([])
+        demo_integration = self.integrateSingleImage(fake_image_to_process)
+        coord_dict.update({'chi':demo_integration.chi,'q':demo_integration.q})
+        npts_q = len(demo_integration.q)
+              
+        order_list = []
+        for idx in indexes:
+            order_list.append(idx)
+            coord_dict[idx] = data.indexes[idx]
+            shape = shape + tuple([len(data.indexes[idx])])
+        shape = shape + (360,npts_q)
+        print(shape)
+        
+        desired_order_list = order_list+['chi','q']
+        coord_dict_sorted = {k: coord_dict[k] for k in desired_order_list}
+        
+        template = xr.DataArray(np.empty(shape),coords=coord_dict_sorted)  
+        template = template.chunk({indexes[0]:chunksize})
+        integ_fly = data.map_blocks(self.integrateImageStack_legacy,template=template)#integ_traditional.chunk({'energy':5}))
+        return integ_fly 
+            
\ No newline at end of file
